//  NLPUtils Project
//  Tokenizer
//  TokenizerTest.cs
//
//  Author: Cody Boisclair
//  Version: 1.0

using System;
using System.Text.RegularExpressions;
using System.Collections.Generic;
using System.Reflection;
using System.IO;

namespace NLPUtils
{
    public class TokenizerTest
    {
        static void Main(string[] args)
        {
            if (args.Length < 1 || (args.Length < 2 && (args[0] == "-f" || args[0] == "-b") ))
            {
                string assemName = Assembly.GetExecutingAssembly().GetName().Name;
                Console.WriteLine(assemName + " - Using Tokenizer version: " + Tokenizer.Version);
                Console.WriteLine();
                Console.WriteLine("Usage:");
                Console.WriteLine("  " + assemName + " [string to analyze]");
                Console.WriteLine("  " + assemName + " -f [filename to analyze]");
                Console.WriteLine("  " + assemName + " -b [Brown Corpus formatted file to analyze]");
                
                Environment.Exit(1);
            }

            string textToAnalyze = null;

            if (args[0] == "-b")
            {
                brownTok(args[1]);
            }
            else
            {
                if (args[0] != "-f")
                {
                    string allArgs = "";
                    for (int i = 0; i < args.Length; i++)
                    {
                        if (i > 0) allArgs += " ";
                        allArgs += args[i];
                    }
                    textToAnalyze = allArgs;
                }
                else
                {
                    TextReader tr = new StreamReader(args[1]);
                    string curLine = tr.ReadLine();
                    while (curLine != null)
                    {
                        textToAnalyze += curLine + " ";
                        curLine = tr.ReadLine();
                    }
                }

                List<Token> toks = Tokenizer.Tokenize("The quick brown fox");

                DateTime startTime = DateTime.Now;
                toks = Tokenizer.Tokenize(textToAnalyze);
                DateTime endTime = DateTime.Now;
                TimeSpan timeSpan = endTime - startTime;

                //Console.WriteLine("Tokens:");
                PrintTokens(toks);
                //Console.WriteLine("Tokenization time: " + timeSpan);
            }
        }

        static void brownTok(string filename)
        {           
            TextReader tr = new StreamReader(filename);
            string accumLines = "";
            string curLine = tr.ReadLine();

            if (curLine.Length < 9 || !Char.IsLetter(curLine[0]) ||
                !Char.IsNumber(curLine[1]))  // not "A0"
            {
                Console.WriteLine("This does not appear to be a Brown Corpus file.");
                Environment.Exit(2);
            }

            while (curLine != null)
            {
                if (curLine.Length < 9)
                {
                    curLine = tr.ReadLine();
                    continue;
                }

                curLine = curLine.Substring(9);  // chop off "A00 0000"
                if (curLine[0] == ' ') // new paragraph
                {
                    if (accumLines != "")
                        brownParaTok(accumLines);
                    accumLines = curLine.Substring(1);
                }
                else
                {
                    accumLines += " " + curLine;
                }
                curLine = tr.ReadLine();
            }
            brownParaTok(accumLines);
        }

        static void brownParaTok(string text)
        {
            // Convert the Brown-style &s representing abbreviations
            // to regular periods
            text = Regex.Replace(text, "&\\.", ".");
            text = Regex.Replace(text, "&", ".");

            // Convert Brown dashes to em dashes
            text = Regex.Replace(text, "-(\\s)", "--$1");

            // Remove the ~s indicating caps
            text = Regex.Replace(text, "~", "");

            List<Token> toks = Tokenizer.Tokenize(text);
            PrintTokens(toks);
            //Console.WriteLine();
        }

        static void PrintTokens(List<Token> list) {
            //Console.Write("[");
            foreach(Token t in list) {
                //if (t != list[0])
                //	Console.Write(" ");
                //Console.Write("\"" + t + "\"");
                Console.WriteLine(t);
            }
            //Console.WriteLine("]");
        }
    }
}
